//===========================================================================
//  @(#) $Name: arts++-1-1-a13 $
//  @(#) $Id: ArtsAsMatrixEntry.hh,v 1.2 2004/04/21 23:51:25 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSASMATRIXENTRY_HH_
#define _ARTSASMATRIXENTRY_HH_

extern "C" {
#include "artslocal.h"
#include <sys/types.h>

#include "caida_t.h"
}

#ifdef HAVE_IOSTREAM
  #include <iostream>
#else
  #include <iostream.h>
#endif

#ifdef HAVE_IOMANIP
  #include <iomanip>
#else
  #include <iomanip.h>
#endif

//---------------------------------------------------------------------------
//  class ArtsAsMatrixEntry
//---------------------------------------------------------------------------
//!  Class representing a single AS matrix entry.  This class holds a
//!  source AS number, a destination AS number, and the packets and bytes
//!  sent from the source AS to the destination AS.
//---------------------------------------------------------------------------
class ArtsAsMatrixEntry
{
public:
  //-------------------------------------------------------------------------
  //                           ArtsAsMatrixEntry()                           
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsAsMatrixEntry();

  //--------------------------------------------------------------------------
  //          ArtsAsMatrixEntry(const ArtsAsMatrixEntry & asmEntry) 
  //..........................................................................
  //!  copy constructor
  //--------------------------------------------------------------------------
  ArtsAsMatrixEntry(const ArtsAsMatrixEntry & asmEntry);
  
  //--------------------------------------------------------------------------
  //                          ~ArtsAsMatrixEntry()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsAsMatrixEntry();

  //-------------------------------------------------------------------------
  //                    inline uint8_t Descriptor() const                    
  //.........................................................................
  //!  Returns the descriptor field of the object.  The descriptor field
  //!  is a bitfield containing indicators of the length of the Src(), Dst(),
  //!  Pkts() and Bytes() fields when stored on disk or sent via a socket.
  //-------------------------------------------------------------------------
  inline uint8_t Descriptor() const
  {
    return(this->_descriptor);
  }

  //-------------------------------------------------------------------------
  //              inline uint8_t Descriptor(uint8_t descriptor)              
  //.........................................................................
  //!  Sets and returns the descriptor field of the object.
  //-------------------------------------------------------------------------
  inline uint8_t Descriptor(uint8_t descriptor)
  {
    this->_descriptor = descriptor;
    return(this->_descriptor);
  }
  
  //-------------------------------------------------------------------------
  //                       inline uint16_t Src() const                       
  //.........................................................................
  //!  Returs the source AS number.
  //-------------------------------------------------------------------------
  inline uint16_t Src() const
  {
    return(this->_src);
  }

  //-------------------------------------------------------------------------
  //                     uint16_t Src(uint16_t src)                    
  //.........................................................................
  //!  Sets and returns the source AS number.
  //-------------------------------------------------------------------------
  uint16_t Src(uint16_t src);

  //-------------------------------------------------------------------------
  //                       inline uint16_t Dst() const                       
  //.........................................................................
  //!  Returns the destination AS number.
  //-------------------------------------------------------------------------
  inline uint16_t Dst() const
  {
    return(this->_dst);
  }

  //-------------------------------------------------------------------------
  //                     uint16_t Dst(uint16_t dst)                    
  //.........................................................................
  //!  Sets and returns the source AS number.
  //-------------------------------------------------------------------------
  uint16_t Dst(uint16_t dst);
  
  //-------------------------------------------------------------------------
  //                       inline uint64_t Pkts() const                      
  //.........................................................................
  //!  Returns the number of packets sent from the source AS to the
  //!  destination AS.
  //-------------------------------------------------------------------------
  inline uint64_t Pkts() const
  {
    return(this->_pkts);
  }

  //-------------------------------------------------------------------------
  //                      uint64_t Pkts(uint64_t pkts)                   
  //.........................................................................
  //!  Sets and returns the number of packets sent from the source AS to the
  //!  destination AS.
  //-------------------------------------------------------------------------
  uint64_t Pkts(uint64_t pkts);
  
  //-------------------------------------------------------------------------
  //                     inline uint64_t Bytes() const                   
  //.........................................................................
  //!  Returns the number of bytes sent from the source AS to the
  //!  destination AS.
  //-------------------------------------------------------------------------
  inline uint64_t Bytes() const
  {
    return(this->_bytes);
  }

  //-------------------------------------------------------------------------
  //                     uint64_t Bytes(uint64_t bytes)                  
  //.........................................................................
  //!  Sets and returns the number of bytes sent from the source AS to the
  //!  destination AS.
  //-------------------------------------------------------------------------
  uint64_t Bytes(uint64_t bytes);

  //-------------------------------------------------------------------------
  //                uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the bytes of space required to store the ArtsAsMatrixEntry
  //!  on disk.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //            std::istream& read(std::istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsAsMatrixEntry from an istream.  Returns the istream.
  //-------------------------------------------------------------------------
  std::istream& read(std::istream& is, uint8_t version = 0);

  //-------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsAsMatrixEntry from a file descriptor.  Returns the
  //!  number of bytes read on success, -1 on failure.
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //          std::ostream& write(std::ostream& os, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsAsMatrixEntry to an ostream.  Returns the ostream.
  //-------------------------------------------------------------------------
  std::ostream& write(std::ostream& os, uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //               int write(int fd, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsAsMatrixEntry to a file descriptor.  Returns the
  //!  number of bytes written on success, -1 on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //   std::ostream & operator << (std::ostream& os,                   
  //                          const ArtsAsMatrixEntry & artsAsMatrixEntry)
  //.........................................................................
  //!  Overloaded '<<' operator to dump the contents of an ArtsAsMatrixEntry
  //!  to an ostream in a human-readable form.  Returns the ostream.
  //-------------------------------------------------------------------------
  friend std::ostream & operator << (std::ostream& os,
                                const ArtsAsMatrixEntry & artsAsMatrixEntry);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                     static uint32_t NumObjects()
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG

private:
  uint8_t             _descriptor;
  uint16_t            _src;
  uint16_t            _dst;
  uint64_t            _pkts;
  uint64_t            _bytes;
  static uint32_t     _numObjects;
};

//---------------------------------------------------------------------------
//  class ArtsAsMatrixEntryGreaterBytes
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting 
//!  ArtsAsMatrixEntry objects in descending order by bytes of traffic. 
//---------------------------------------------------------------------------
class ArtsAsMatrixEntryGreaterBytes
{
public:
  //---------------------------------------------------------------------------
  //    bool operator () (const ArtsAsMatrixEntry & asEntry1,
  //                      const ArtsAsMatrixEntry & asEntry2) const
  //...........................................................................
  //
  //---------------------------------------------------------------------------
  bool operator () (const ArtsAsMatrixEntry & asEntry1,
                    const ArtsAsMatrixEntry & asEntry2) const;
};

//---------------------------------------------------------------------------
//  class ArtsAsMatrixEntryGreaterPkts
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting
//!  ArtsAsMatrixEntry objects in descending order by packets of traffic.
//---------------------------------------------------------------------------
class ArtsAsMatrixEntryGreaterPkts
{
public:
  //---------------------------------------------------------------------------
  //    bool operator () (const ArtsAsMatrixEntry & asEntry1,
  //                      const ArtsAsMatrixEntry & asEntry2) const
  //...........................................................................
  //
  //---------------------------------------------------------------------------
  bool operator () (const ArtsAsMatrixEntry & asEntry1,
                    const ArtsAsMatrixEntry & asEntry2) const;
};

#endif  /*  _ARTSASMATRIXENTRY_HH_  */
