//===========================================================================
//  $Name: arts++-1-1-a13 $
//  $Id: ArtsNextHopTableEntry.hh,v 1.2 2004/04/21 23:51:27 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSNEXTHOPTABLEENTRY_HH_
#define _ARTSNEXTHOPTABLEENTRY_HH_

extern "C" {
#include "artslocal.h"
#include <sys/types.h>
#include "caida_t.h"
}

#ifdef HAVE_IOSTREAM
  #include <iostream>
#else
  #include <iostream.h>
#endif

#ifdef HAVE_IOMANIP
  #include <iomanip>
#else
  #include <iomanip.h>
#endif

//---------------------------------------------------------------------------
//  class ArtsNextHopTableEntry
//---------------------------------------------------------------------------
//!  Class representing a NextHop table entry.  This class is used to hold
//!  traffic counters (number of packets and bytes) per IP NextHop.
//---------------------------------------------------------------------------
class ArtsNextHopTableEntry
{
public:
  //-------------------------------------------------------------------------
  //                       ArtsNextHopTableEntry()
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsNextHopTableEntry();

  //--------------------------------------------------------------------------
  //    ArtsNextHopTableEntry(const ArtsNextHopTableEntry & nexthopEntry) 
  //..........................................................................
  //!  copy covnstructor
  //--------------------------------------------------------------------------
  ArtsNextHopTableEntry(const ArtsNextHopTableEntry & nexthopEntry);
  
  //-------------------------------------------------------------------------
  //                       ~ArtsNextHopTableEntry()
  //.........................................................................
  //!  destructor
  //-------------------------------------------------------------------------
  ~ArtsNextHopTableEntry();

  //-------------------------------------------------------------------------
  //                    ipv4addr_t IpAddr() const
  //.........................................................................
  //!  Returns the IP address of the NextHop entry.
  //-------------------------------------------------------------------------
  ipv4addr_t IpAddr() const;
  
  //-------------------------------------------------------------------------
  //                ipv4addr_t IpAddr(ipv4addr_t ipAddr)
  //.........................................................................
  //!  Sets and returns the IP address for the NextHop entry.
  //-------------------------------------------------------------------------
  ipv4addr_t IpAddr(ipv4addr_t ipAddr);
  
  //-------------------------------------------------------------------------
  //                       uint64_t Pkts() const
  //.........................................................................
  //!  Returns the packet count for the NextHop entry.
  //-------------------------------------------------------------------------
  uint64_t Pkts() const;
  
  //-------------------------------------------------------------------------
  //                    uint64_t Pkts(uint64_t pkts)
  //.........................................................................
  //!  Sets and returns the packet count for the NextHop entry.
  //-------------------------------------------------------------------------
  uint64_t Pkts(uint64_t pkts);
  
  //-------------------------------------------------------------------------
  //                      inline uint64_t Bytes() const
  //.........................................................................
  //!  Returns the byte count for the NextHop entry.
  //-------------------------------------------------------------------------
  uint64_t Bytes() const;
  
  //-------------------------------------------------------------------------
  //                  inline uint64_t Bytes(uint64_t bytes)
  //.........................................................................
  //!  Sets and returns the byte count for the NextHop entry.
  //-------------------------------------------------------------------------
  uint64_t Bytes(uint64_t bytes);

  //-------------------------------------------------------------------------
  //                uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the number of bytes needed to store the NextHop entry in a
  //!  file.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //                       uint8_t Descriptor() const
  //.........................................................................
  //!  Returns the descriptor field.
  //-------------------------------------------------------------------------
  uint8_t Descriptor() const;

  //-------------------------------------------------------------------------
  //                 uint8_t Descriptor(uint8_t descriptor)
  //.........................................................................
  //!  Sets and returns the descriptor field.
  //-------------------------------------------------------------------------
  uint8_t Descriptor(uint8_t descriptor);

  //-------------------------------------------------------------------------
  //            std::istream& read(std::istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsNextHopTableEntry from an istream.  Returns the istream.
  //-------------------------------------------------------------------------
  std::istream& read(std::istream& is, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //                 int read(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsNextHopTableEntry from a file descriptor.  Returns the
  //!  number of bytes read on success, -1 on failure.
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //          std::ostream& write(std::ostream& os, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsNextHopTableEntry to an ostream.  Returns the ostream.
  //-------------------------------------------------------------------------
  std::ostream& write(std::ostream& os, uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //               int write(int fd, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsNextHopTableEntry to a file descriptor.  Returns the
  //!  number of bytes written on success, -1 on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //   ArtsNextHopTableEntry &                        
  //   operator = (const ArtsNextHopTableEntry & artsNextHopTableEntry)
  //.........................................................................
  //!  Overloaded '=' operator to assign one ArtsNextHopTableEntry to
  //!  another.
  //-------------------------------------------------------------------------
  ArtsNextHopTableEntry &
  operator = (const ArtsNextHopTableEntry & artsNextHopTableEntry);

  //-------------------------------------------------------------------------
  //   friend std::ostream &                            
  //   operator << (std::ostream& os,                   
  //                const ArtsNextHopTableEntry & artsNextHopTableEntry)
  //.........................................................................
  //!  Overloaded '<<' operator to dump the contents of an
  //!  ArtsNextHopTableEntry to an ostream in a human-readable form.
  //!  Returns the ostream.
  //-------------------------------------------------------------------------
  friend std::ostream &
  operator << (std::ostream& os,
               const ArtsNextHopTableEntry & artsNextHopTableEntry);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                       static uint32_t NumObjects()                      
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG
  
private:
  ipv4addr_t  _ipAddr;
  uint8_t     _descriptor;
  uint64_t    _pkts;
  uint64_t    _bytes;

  static uint32_t  _numObjects;
};

//---------------------------------------------------------------------------
//  class ArtsNextHopEntryGreaterBytes
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting 
//!  ArtsNextHopTableEntry objects in descending order by bytes of
//!  traffic. 
//---------------------------------------------------------------------------
class ArtsNextHopEntryGreaterBytes
{
public:
  bool operator()(const ArtsNextHopTableEntry & nextHopEntry1,
                  const ArtsNextHopTableEntry & nextHopEntry2) const;
};


//---------------------------------------------------------------------------
//  class ArtsNextHopEntryGreaterPkts
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting 
//!  ArtsNextHopTableEntry objects in descending order by packets of
//!  traffic. 
//---------------------------------------------------------------------------
class ArtsNextHopEntryGreaterPkts
{
public:
  bool operator()(const ArtsNextHopTableEntry & nextHopEntry1,
                  const ArtsNextHopTableEntry & nextHopEntry2) const;
};


#endif  // _ARTSNEXTHOPTABLEENTRY_HH_
